//
// server.cpp
// ~~~~~~~~~~
//
// Copyright (c) 2015 Fabien Rico (fabien dot rico at univ-lyon1.fr)
//
// Distributed under the GPL License, Version 1.0.

#include <ctime>
#include <iostream>
#include <string>
#include <boost/asio.hpp>
#include <boost/array.hpp>

using namespace std;

// comme le programme utilisera TCP/IP, on fait correspondre tcp avec
// boost::asio::ip::tcp afin d'alléger le code
using boost::asio::ip::tcp;

int main(int argc, char* argv[])
{
  if (argc != 2) {
    cerr << "Usage " << argv[0] << " <numport>" << endl;
    exit(1);
  }

  try {
    boost::system::error_code error;
    // Lien avec le système d'exploitation
    boost::asio::io_service service;

    // on cree fait appel au système pour trouver une ip et un port
    // en fonction de la demande
    tcp::resolver resolver(service);
    tcp::resolver::query query(argv[1]);

    // on recherche les adresses correspondantes
    tcp::resolver::iterator iterator = resolver.resolve(query);

    // on parcours la liste des résultats
    tcp::acceptor acceptor(service);
    tcp::resolver::iterator end;
    // creation d'une écoute sur le port demandé
    while(iterator != end) {
      tcp::endpoint endpoint = *iterator++;

      cout << "Essaye " << endpoint << "...";
      cout.flush();
      acceptor.open(endpoint.protocol());

      acceptor.set_option(tcp::acceptor::reuse_address(true));
      acceptor.bind(endpoint, error);
      if (!error) {
	cout << "réussie !" << endl;
	break;
      }
      cout << "impossible car " << error.message() << endl;
      acceptor.close();
    }
    if (iterator==end) {
      cout << "Impossible de se connecter" << endl;
      throw boost::system::system_error(error);
    }
    acceptor.listen();
    
    // accueil d'un client
    tcp::socket socket(service);

    acceptor.accept(socket);

    cout << "Création d'une nouvelle connexion entre "
	 <<  socket.local_endpoint()
	 << " et " << socket.remote_endpoint() << endl;
    
    // envoie d'un premier message
    boost::asio::write(socket,
		       boost::asio::buffer("Welcom\r\n"),
		       error);
    if (error) {
      cerr << "Il y a un problème d'écriture sur la socket" << error << endl;
      throw boost::system::system_error(error);
    }

    while (1) {
      // lecture 5 carractères à la fois
      boost::array<char, 5> buf;
      size_t len = socket.read_some(boost::asio::buffer(buf), error);
      if (error == boost::asio::error::eof){
	cout << "fermeture de la socket par le client" << endl;
	break;
      }
      if (error) {
        cerr << "Il y a un problème de lecture sur la socket" << endl;
        throw boost::system::system_error(error);
      }
      string s(buf.data(), len);

      cout << "recu : '" << s << "'" << endl;
    }
    socket.close();
  }
  catch (exception &e) {
    cerr << "il y a eu une exception " << e.what() << endl;
  }

  return 0;
}

